searchData={"content_type":"text/markdown","items":[{"doc":"","ref":"QueryBuilder.html","title":"QueryBuilder","type":"module"},{"doc":"","ref":"QueryBuilder.html#default_page_size/0","title":"QueryBuilder.default_page_size/0","type":"function"},{"doc":"Allows to pass a list of operations through a keyword list.\r\n\r\nExample:\r\n```\r\nQueryBuilder.from_list(query, [\r\n  where: [name: \"John\", city: \"Anytown\"],\r\n  preload: [articles: :comments]\r\n])\r\n```","ref":"QueryBuilder.html#from_list/2","title":"QueryBuilder.from_list/2","type":"function"},{"doc":"A join query expression.\r\n\r\nExample:\r\n```\r\nQueryBuilder.left_join(query, :articles, title@articles: \"Foo\", or: [title@articles: \"Bar\"])\r\n```","ref":"QueryBuilder.html#left_join/4","title":"QueryBuilder.left_join/4","type":"function"},{"doc":"A limit query expression.\r\nIf multiple limit expressions are provided, the last expression is evaluated\r\n\r\nExample:\r\n```\r\nQueryBuilder.limit(query, 10)\r\n```","ref":"QueryBuilder.html#limit/2","title":"QueryBuilder.limit/2","type":"function"},{"doc":"Run `QueryBuilder.where/2` only if given condition is met.","ref":"QueryBuilder.html#maybe_where/3","title":"QueryBuilder.maybe_where/3","type":"function"},{"doc":"Run `QueryBuilder.where/4` only if given condition is met.","ref":"QueryBuilder.html#maybe_where/5","title":"QueryBuilder.maybe_where/5","type":"function"},{"doc":"","ref":"QueryBuilder.html#new/1","title":"QueryBuilder.new/1","type":"function"},{"doc":"A offset query expression.\r\nIf multiple offset expressions are provided, the last expression is evaluated\r\n\r\nExample:\r\n```\r\nQueryBuilder.offset(query, 10)\r\n```","ref":"QueryBuilder.html#offset/2","title":"QueryBuilder.offset/2","type":"function"},{"doc":"An order by query expression.\r\n\r\nExample:\r\n```\r\nQueryBuilder.order_by(query, asc: :lastname, asc: :firstname)\r\n```","ref":"QueryBuilder.html#order_by/2","title":"QueryBuilder.order_by/2","type":"function"},{"doc":"An order by query expression.\r\n\r\nFor more about the second argument, see `where/3`.\r\n\r\nExample:\r\n```\r\nQueryBuilder.order_by(query, :articles, asc: :title@articles)\r\n```","ref":"QueryBuilder.html#order_by/3","title":"QueryBuilder.order_by/3","type":"function"},{"doc":"","ref":"QueryBuilder.html#paginate/3","title":"QueryBuilder.paginate/3","type":"function"},{"doc":"Preloads the associations.\r\n\r\nBindings are automatically set if joins have been made, or if it is preferable to\r\njoin (i.e. one-to-one associations are preferable to include into the query result\r\nrather than emitting separate DB queries).\r\n\r\nExample:\r\n```\r\nQueryBuilder.preload(query, [role: :permissions, articles: [:stars, comments: :user]])\r\n```","ref":"QueryBuilder.html#preload/2","title":"QueryBuilder.preload/2","type":"function"},{"doc":"An AND where query expression.\r\n\r\nExample:\r\n```\r\nQueryBuilder.where(query, firstname: \"John\")\r\n```","ref":"QueryBuilder.html#where/2","title":"QueryBuilder.where/2","type":"function"},{"doc":"An AND where query expression.\r\n\r\nAssociations are passed in second argument; fields from these associations can then\r\nbe referenced by writing the field name, followed by the \"@\" character and the\r\nassociation name, as an atom. For example: `:name@users`.\r\n\r\nExample:\r\n```\r\nQueryBuilder.where(query, [role: :permissions], name@permissions: :write)\r\n```\r\n\r\nOR clauses may be passed through last argument `opts`. For example:\r\n\r\n```elixir\r\nQueryBuilder.where(query, [], [firstname: \"John\"], or: [firstname: \"Alice\", lastname: \"Doe\"], or: [firstname: \"Bob\"])\r\n```","ref":"QueryBuilder.html#where/4","title":"QueryBuilder.where/4","type":"function"},{"doc":"Use this module to create an extension module to `QueryBuilder` for app specific query utilities.\r\nUse your query builder extension module wherever you would normally use `QueryBuilder`\r\n\r\nExample:\r\n```\r\ndefmodule MyApp.QueryBuilder do\r\n  use QueryBuilder.Extension\r\n\r\n  defmacro __using__(opts) do\r\n    quote do\r\n      require QueryBuilder\r\n      QueryBuilder.__using__(unquote(opts))\r\n    end\r\n  end\r\n\r\n  # Add app specific query functions\r\n  #---------------------------------\r\n\r\n  def where_initcap(query, field, value) do\r\n    text_equals_condition = fn field, value, get_binding_fun ->\r\n      {field, binding} = get_binding_fun.(field)\r\n      Ecto.Query.dynamic([{^binding, x}], fragment(\"initcap(?)\", ^value) == field(x, ^field))\r\n    end\r\n\r\n    query\r\n    |> where(&text_equals_condition.(field, value, &1))\r\n  end\r\nend\r\n\r\ndefmodule MyApp.Accounts.User do\r\n  use MyApp.QueryBuilder\r\n\r\n  schema \"users\" do\r\n    field :name, :string\r\n    field :active, :boolean\r\n  end\r\nend\r\n\r\ndefmodule MyApp.Accounts do\r\n  alias MyApp.QueryBuilder, as: QB\r\n\r\n  def list_users(opts \\\\ []) do\r\n    # Query list can include custom query functions as well:\r\n    # [where_initcap: {:name, \"john\"}, where: {:active, true}]\r\n    MyApp.Accounts.User\r\n    |> QB.from_list(opts)\r\n    |> Repo.all()\r\n  end\r\nend\r\n```","ref":"QueryBuilder.Extension.html","title":"QueryBuilder.Extension","type":"module"},{"doc":"# Query Builder\r\n\r\nQuery Builder allows you to build and compose Ecto queries based on data.\r\n\r\n```elixir\r\nUser\r\n|> QueryBuilder.where(firstname: \"John\")\r\n|> QueryBuilder.where([{:age, :gt, 30}, city: \"Anytown\"])\r\n|> QueryBuilder.order_by(asc: :lastname)\r\n|> QueryBuilder.preload([:role, authored_articles: :comments])\r\n|> QueryBuilder.offset(20)\r\n|> QueryBuilder.limit(10)\r\n|> Repo.all()\r\n```\r\n\r\nThis allows writing queries more concisely, without having to deal with bindings\r\nand macros.\r\n\r\nIts primary goal is to allow Context functions to receive a set of filters and\r\noptions:\r\n\r\n```elixir\r\nBlog.list_articles(preload: :comments, order_by: [asc: :title])\r\nBlog.list_articles(preload: [:category, comments: :user])\r\n```\r\n\r\nThis avoids having to create many different functions in the Context for every\r\ncombination of filters and options, or to create one general function that does\r\ntoo much to satisfy all the consumers.\r\n\r\nThe calling code (e.g. the Controllers), can now retrieve the list of articles with\r\ndifferent options. In some part of the application, the category is needed; in other\r\nparts it is not; sometimes the articles must be sorted based on their title;\r\nother times it doesn't matter, etc.\r\n\r\nSee `QueryBuilder.from_list/2` below.","ref":"readme.html","title":"Query Builder","type":"extras"},{"doc":"```elixir\r\nUser\r\n|> QueryBuilder.where(firstname: \"John\")\r\n|> QueryBuilder.where([{:age, :gt, 30}, city: \"Anytown\"])\r\n|> QueryBuilder.order_by(asc: :lastname)\r\n|> QueryBuilder.preload([:role, authored_articles: :comments])\r\n|> QueryBuilder.offset(20)\r\n|> QueryBuilder.limit(10)\r\n|> Repo.all()\r\n```\r\n\r\nFiltering on associations is supported:\r\n\r\n```elixir\r\nUser\r\n|> QueryBuilder.where(:role, name@role: \"admin\")\r\n|> Repo.all()\r\n```\r\n\r\n```elixir\r\nUser\r\n|> QueryBuilder.where([role: :permissions], name@permissions: \"delete\")\r\n|> Repo.all()\r\n```\r\n\r\n```elixir\r\nArticle\r\n|> QueryBuilder.where(:author, id@author: author_id)\r\n|> QueryBuilder.where([:author, :comments], {:logged_at@author, :lt, :inserted_at@comments})\r\n|> QueryBuilder.preload(:comments)\r\n|> Repo.all()\r\n```","ref":"readme.html#examples","title":"Examples - Query Builder","type":"extras"},{"doc":"Add `use QueryBuilder` in your schema:\r\n\r\n```elixir\r\ndefmodule MyApp.User do\r\n  use Ecto.Schema\r\n  use QueryBuilder\r\n\r\n  schema \"users\" do\r\n    # code\r\n  end\r\n\r\n  # code\r\nend\r\n```\r\n\r\nYou may also specify the schema's associations to `QueryBuilder` in order to remedy\r\nsome limitations when building queries:\r\n\r\n```elixir\r\ndefmodule MyApp.User do\r\n  use Ecto.Schema\r\n  use QueryBuilder, assoc_fields: [:role, :articles]\r\n\r\n  schema \"users\" do\r\n    # code\r\n    belongs_to :role, MyApp.Role\r\n    has_many :articles, MyApp.Article\r\n  end\r\n\r\n  # code\r\nend\r\n```\r\n\r\nCurrently, supported operations are:\r\n\r\n`QueryBuilder.paginate/3`\r\n\r\n```elixir\r\n%{\r\n  cursor_direction: :after,\r\n  cursor_for_entries_before: cursor_for_entries_before,\r\n  cursor_for_entries_after: cursor_for_entries_after,\r\n  has_more_entries: has_more_entries?\r\n} =\r\n  User\r\n  |> QueryBuilder.order_by(asc: :nickname, desc: :email)\r\n  |> QueryBuilder.paginate(Repo, page_size: 10)\r\n\r\n# for subsequent request you may use the cursor:\r\n\r\nUser\r\n|> QueryBuilder.order_by(asc: :nickname, desc: :email)\r\n|> QueryBuilder.paginate(Repo, page_size: 10, cursor: cursor_for_entries_after, direction: :after)\r\n```\r\n\r\nYou may configure a global default page size via config:\r\n\r\n```elixir\r\nconfig :query_builder, :default_page_size, 100\r\n```\r\n\r\n`QueryBuilder.where/2`\r\n\r\n```elixir\r\nQueryBuilder.where(query, firstname: \"John\")\r\n```\r\n\r\n`QueryBuilder.where/4`\r\n\r\n```elixir\r\nQueryBuilder.where(query, [role: :permissions], name@permissions: :write)\r\n```\r\n\r\nAbove `where` functions support different filter operations, for instance:\r\n```elixir\r\nQueryBuilder.where(query, {:age, :greater_than, 18})\r\n```\r\n\r\nSupported filter operations are:\r\n* `:equal_to` (or `:eq`)\r\n* `:other_than` (or `:ne`)\r\n* `:greater_than` (or `:gt`)\r\n* `:greater_than_or_equal_to` (or `:ge`)\r\n* `:less_than` (or `:lt`)\r\n* `:less_than_or_equal_to` (or `:le`)\r\n* `:like`\r\n* `:ilike`\r\n* `:starts_with`\r\n* `:ends_with`\r\n* `:contains`\r\n\r\nArray inclusion checking:\r\n* `:in`\r\n* `:not_in`\r\n* `:include`\r\n* `:exclude`\r\n\r\nNote that `:starts_with`, `:ends_with` and `:contains` operations can be written using `:like`, but offer a more declarative style and are safer, as they escape the `%` and `_` characters for you. You may also perform case insensitive searchs using these functions. Example:\r\n\r\n```elixir\r\nQueryBuilder.where({:name, :starts_with, \"jo\"})\r\n```\r\n\r\n```elixir\r\nQueryBuilder.where({:name, :starts_with, \"jo\", case: :insensitive}) # `:i` will also work\r\n```\r\n\r\nWhen using `:like` or `:ilike`, make sure to escape `%` and `_` characters properly.\r\n\r\nYou may also add `OR` clauses through `QueryBuilder.where/4`'s fourth argument:\r\n\r\n```elixir\r\nQueryBuilder.where(query, [], [name: \"John\"], or: [name: \"Alice\", age: 42], or: [name: \"Bob\"])\r\n```\r\n\r\n`QueryBuilder.maybe_where/3` and `QueryBuilder.maybe_where/5`\r\n\r\n```elixir\r\nquery\r\n|> QueryBuilder.maybe_where(some_condition, name: \"Alice\")\r\n```\r\n\r\nThe above will run `where/2` if the given condition is met.\r\n\r\n`QueryBuilder.order_by/2`\r\n\r\n```elixir\r\nQueryBuilder.order_by(query, asc: :lastname, asc: :firstname)\r\n```\r\n\r\n`QueryBuilder.order_by/3`\r\n\r\n```elixir\r\nQueryBuilder.order_by(query, :articles, asc: :title@articles)\r\n```\r\n\r\n`QueryBuilder.preload/2`\r\n\r\n```elixir\r\nQueryBuilder.preload(query, [role: :permissions, articles: [:stars, comments: :user]])\r\n```\r\n\r\n`QueryBuilder.left_join/4`\r\n\r\n```elixir\r\nQueryBuilder.left_join(query, :articles, title@articles: \"Foo\", or: [title@articles: \"Bar\"])\r\n```\r\n\r\n`QueryBuilder.offset/2`\r\n\r\n```elixir\r\nQueryBuilder.offset(query, 10)\r\n```\r\n\r\n`QueryBuilder.limit/2`\r\n\r\n```elixir\r\nQueryBuilder.limit(query, 10)\r\n```\r\n\r\n`QueryBuilder.from_list/2`\r\n\r\n```elixir\r\nQueryBuilder.from_list(query, [\r\n  where: [name: \"John\", city: \"Anytown\"],\r\n  preload: [articles: :comments],\r\n  order_by: {:articles, asc: :title@articles},\r\n  limit: 20,\r\n  offset: 10\r\n])\r\n```\r\n\r\nThe `QueryBuilder.from_list/2` function was the main motivation behind the writing\r\nof this library. As explained above, it allows to add querying options to the\r\nContext functions. Example:\r\n\r\n```elixir\r\ndefmodule MyApp.Blog do\r\n  alias MyApp.Blog.Article\r\n\r\n  def get_article_by_id(id, opts \\\\ []) do\r\n    QueryBuilder.where(Article, id: id)\r\n    |> QueryBuilder.from_list(opts)\r\n    |> Repo.one!()\r\n  end\r\nend\r\n```\r\n\r\nThe function can now be called as follows (for instance, from a Controller):\r\n\r\n```elixir\r\nBlog.get_article_by_id(id, preload: [:comments])\r\n\r\nBlog.get_article_by_id(id, preload: [:likes])\r\n\r\nBlog.get_article_by_id(\r\n  id,\r\n  order_by: {:comments, desc: :inserted_at@comments},\r\n  preload: [comments: :user]\r\n)\r\n```","ref":"readme.html#usage","title":"Usage - Query Builder","type":"extras"},{"doc":"In the event that you want to extend QueryBuilder's functionality to include custom app specific query functions, there's the `QueryBuilder.Extension` module to facilitate that.  You can create a module with your app specific query functionality and `use` the `QueryBuilder.Extension` module to inject all `QueryBuilder` functions into your custom module.  Any custom query functions added to your custom module are included in `QueryBuilder.from_list/2`.\r\n\r\nFor example:\r\n\r\n```elixir\r\ndefmodule MyApp.QueryBuilder do\r\n  use QueryBuilder.Extension\r\n\r\n  defmacro __using__(opts) do\r\n    quote do\r\n      require QueryBuilder\r\n      QueryBuilder.__using__(unquote(opts))\r\n    end\r\n  end\r\n\r\n  # Add app specific query functions here...\r\n\r\n  def where_initcap(query, field, value) do\r\n    text_equals_condition = fn field, value, get_binding_fun ->\r\n      {field, binding} = get_binding_fun.(field)\r\n      Ecto.Query.dynamic([{^binding, x}], fragment(\"initcap(?)\", ^value) == field(x, ^field))\r\n    end\r\n\r\n    query\r\n    |> where(&text_equals_condition.(field, value, &1))\r\n  end\r\nend\r\n\r\ndefmodule MyApp.Accounts.User do\r\n  use MyApp.QueryBuilder\r\n\r\n  schema \"users\" do\r\n    field :name, :string\r\n    field :active, :boolean\r\n  end\r\nend\r\n\r\ndefmodule MyApp.Accounts do\r\n  alias MyApp.QueryBuilder, as: Query\r\n\r\n  def list_users(opts \\\\ []) do\r\n    # Query list can include custom query functions as well:\r\n    # [where_initcap: {:name, \"john\"}, where: {:active, true}]\r\n    MyApp.Accounts.User\r\n    |> Query.from_list(opts)\r\n    |> Repo.all()\r\n  end\r\nend\r\n```","ref":"readme.html#extending-query-functions","title":"Extending Query Functions - Query Builder","type":"extras"},{"doc":"With the auto-binding functionality offered by Query Builder, you can specify field comparisons in queries using the special atom value syntax: `: @self`. This way Query Builder understands the intent is to compare fields vs a raw value.  For example:\r\n\r\n```elixir\r\nusers_where_name_matches_nickname =\r\n  User\r\n  |> QueryBuilder.where({:name, :eq, :nickname@self})\r\n  |> Repo.all()\r\n```\r\n\r\nwould resolve to a query along the following form:\r\n\r\n```sql\r\nselect * from users where users.name = users.nickname\r\n```","ref":"readme.html#special-considerations","title":"Special Considerations - Query Builder","type":"extras"},{"doc":"Add `query_builder` for Elixir as a dependency in your `mix.exs` file:\r\n\r\n```elixir\r\ndef deps do\r\n  [\r\n    {:query_builder, \"~> 1.2.0\"}\r\n  ]\r\nend\r\n```","ref":"readme.html#installation","title":"Installation - Query Builder","type":"extras"},{"doc":"HexDocs documentation can be found at [https://hexdocs.pm/query_builder](https://hexdocs.pm/query_builder).","ref":"readme.html#hexdocs","title":"HexDocs - Query Builder","type":"extras"}]}