searchNodes=[{"doc":"","ref":"QueryBuilder.html","title":"QueryBuilder","type":"module"},{"doc":"Allows to pass a list of operations through a keyword list. Example: QueryBuilder . from_list ( query , [ where : [ name : &quot;John&quot; , city : &quot;Anytown&quot; ] , preload : [ articles : :comments ] ] )","ref":"QueryBuilder.html#from_list/2","title":"QueryBuilder.from_list/2","type":"function"},{"doc":"A join query expression. Example: QueryBuilder . left_join ( query , :articles , title@articles : &quot;Foo&quot; , or : [ title@articles : &quot;Bar&quot; ] )","ref":"QueryBuilder.html#left_join/4","title":"QueryBuilder.left_join/4","type":"function"},{"doc":"A limit query expression. If multiple limit expressions are provided, the last expression is evaluated Example: QueryBuilder . limit ( query , 10 )","ref":"QueryBuilder.html#limit/2","title":"QueryBuilder.limit/2","type":"function"},{"doc":"Run QueryBuilder.where/2 only if given condition is met.","ref":"QueryBuilder.html#maybe_where/3","title":"QueryBuilder.maybe_where/3","type":"function"},{"doc":"Run QueryBuilder.where/4 only if given condition is met.","ref":"QueryBuilder.html#maybe_where/5","title":"QueryBuilder.maybe_where/5","type":"function"},{"doc":"","ref":"QueryBuilder.html#new/1","title":"QueryBuilder.new/1","type":"function"},{"doc":"A offset query expression. If multiple offset expressions are provided, the last expression is evaluated Example: QueryBuilder . offset ( query , 10 )","ref":"QueryBuilder.html#offset/2","title":"QueryBuilder.offset/2","type":"function"},{"doc":"An order by query expression. Example: QueryBuilder . order_by ( query , asc : :lastname , asc : :firstname )","ref":"QueryBuilder.html#order_by/2","title":"QueryBuilder.order_by/2","type":"function"},{"doc":"An order by query expression. For more about the second argument, see where/3 . Example: QueryBuilder . order_by ( query , :articles , asc : :title@articles )","ref":"QueryBuilder.html#order_by/3","title":"QueryBuilder.order_by/3","type":"function"},{"doc":"Preloads the associations. Bindings are automatically set if joins have been made, or if it is preferable to join (i.e. one-to-one associations are preferable to include into the query result rather than emitting separate DB queries). Example: QueryBuilder . preload ( query , [ role : :permissions , articles : [ :stars , comments : :user ] ] )","ref":"QueryBuilder.html#preload/2","title":"QueryBuilder.preload/2","type":"function"},{"doc":"An AND where query expression. Example: QueryBuilder . where ( query , firstname : &quot;John&quot; )","ref":"QueryBuilder.html#where/2","title":"QueryBuilder.where/2","type":"function"},{"doc":"An AND where query expression. Associations are passed in second argument; fields from these associations can then be referenced by writing the field name, followed by the &quot;@&quot; character and the association name, as an atom. For example: :name@users . Example: QueryBuilder . where ( query , [ role : :permissions ] , name@permissions : :write ) OR clauses may be passed through last argument opts . For example: QueryBuilder . where ( query , [ ] , [ firstname : &quot;John&quot; ] , or : [ firstname : &quot;Alice&quot; , lastname : &quot;Doe&quot; ] , or : [ firstname : &quot;Bob&quot; ] )","ref":"QueryBuilder.html#where/4","title":"QueryBuilder.where/4","type":"function"},{"doc":"Use this module to create an extension module to QueryBuilder for app specific query utilities. Use your query builder extension module wherever you would normally use QueryBuilder Example: defmodule MyApp.QueryBuilder do use QueryBuilder.Extension defmacro __using__ ( opts ) do quote do require QueryBuilder QueryBuilder . __using__ ( unquote ( opts ) ) end end # Add app specific query functions #--------------------------------- def where_initcap ( query , field , value ) do text_equals_condition = fn field , value , get_binding_fun -&gt; { field , binding } = get_binding_fun . ( field ) Ecto.Query . dynamic ( [ { ^ binding , x } ] , fragment ( &quot;initcap(?)&quot; , ^ value ) == field ( x , ^ field ) ) end query |&gt; where ( &amp; text_equals_condition . ( field , value , &amp;1 ) ) end end defmodule MyApp.Accounts.User do use MyApp.QueryBuilder schema &quot;users&quot; do field :name , :string field :active , :boolean end end defmodule MyApp.Accounts do alias MyApp.QueryBuilder , as : QB def list_users ( opts \\\\ [ ] ) do # Query list can include custom query functions as well: # [where_initcap: {:name, &quot;john&quot;}, where: {:active, true}] MyApp.Accounts.User |&gt; QB . from_list ( opts ) |&gt; Repo . all ( ) end end","ref":"QueryBuilder.Extension.html","title":"QueryBuilder.Extension","type":"module"},{"doc":"Query Builder Query Builder allows you to build and compose Ecto queries based on data. User |&gt; QueryBuilder . where ( firstname : &quot;John&quot; ) |&gt; QueryBuilder . where ( [ { :age , :gt , 30 } , city : &quot;Anytown&quot; ] ) |&gt; QueryBuilder . order_by ( asc : :lastname ) |&gt; QueryBuilder . preload ( [ :role , authored_articles : :comments ] ) |&gt; QueryBuilder . offset ( 20 ) |&gt; QueryBuilder . limit ( 10 ) |&gt; Repo . all ( ) This allows writing queries more concisely, without having to deal with bindings and macros. Its primary goal is to allow Context functions to receive a set of filters and options: Blog . list_articles ( preload : :comments , order_by : [ asc : :title ] ) Blog . list_articles ( preload : [ :category , comments : :user ] ) This avoids having to create many different functions in the Context for every combination of filters and options, or to create one general function that does too much to satisfy all the consumers. The calling code (e.g. the Controllers), can now retrieve the list of articles with different options. In some part of the application, the category is needed; in other parts it is not; sometimes the articles must be sorted based on their title; other times it doesn't matter, etc. See QueryBuilder.from_list/2 below.","ref":"readme.html","title":"Query Builder","type":"extras"},{"doc":"User |&gt; QueryBuilder . where ( firstname : &quot;John&quot; ) |&gt; QueryBuilder . where ( [ { :age , :gt , 30 } , city : &quot;Anytown&quot; ] ) |&gt; QueryBuilder . order_by ( asc : :lastname ) |&gt; QueryBuilder . preload ( [ :role , authored_articles : :comments ] ) |&gt; QueryBuilder . offset ( 20 ) |&gt; QueryBuilder . limit ( 10 ) |&gt; Repo . all ( ) Filtering on associations is supported: User |&gt; QueryBuilder . where ( :role , name@role : &quot;admin&quot; ) |&gt; Repo . all ( ) User |&gt; QueryBuilder . where ( [ role : :permissions ] , name@permissions : &quot;delete&quot; ) |&gt; Repo . all ( ) Article |&gt; QueryBuilder . where ( :author , id@author : author_id ) |&gt; QueryBuilder . where ( [ :author , :comments ] , { :logged_at@author , :lt , :inserted_at@comments } ) |&gt; QueryBuilder . preload ( :comments ) |&gt; Repo . all ( )","ref":"readme.html#examples","title":"Query Builder - Examples","type":"extras"},{"doc":"Add use QueryBuilder in your schema: defmodule MyApp.User do use Ecto.Schema use QueryBuilder schema &quot;users&quot; do # code end # code end You may also specify the schema's associations to QueryBuilder in order to remedy some limitations when building queries: defmodule MyApp.User do use Ecto.Schema use QueryBuilder , assoc_fields : [ :role , :articles ] schema &quot;users&quot; do # code belongs_to :role , MyApp.Role has_many :articles , MyApp.Article end # code end Currently, supported operations are: QueryBuilder.where/2 QueryBuilder . where ( query , firstname : &quot;John&quot; ) QueryBuilder.where/4 QueryBuilder . where ( query , [ role : :permissions ] , name@permissions : :write ) Above where functions support different filter operations, for instance: QueryBuilder . where ( query , { :age , :greater_than , 18 } ) Supported filter operations are: :equal_to (or :eq ) :other_than (or :ne ) :greater_than (or :gt ) :greater_than_or_equal_to (or :ge ) :less_than (or :lt ) :less_than_or_equal_to (or :le ) :like :ilike :starts_with :ends_with :contains Array inclusion checking: :in :not_in :include :exclude Note that :starts_with , :ends_with and :contains operations can be written using :like , but offer a more declarative style and are safer, as they escape the % and _ characters for you. You may also perform case insensitive searchs using these functions. Example: QueryBuilder . where ( { :name , :starts_with , &quot;jo&quot; } ) QueryBuilder . where ( { :name , :starts_with , &quot;jo&quot; , case : :insensitive } ) # `:i` will also work When using :like or :ilike , make sure to escape % and _ characters properly. You may also add OR clauses through QueryBuilder.where/4 's fourth argument: QueryBuilder . where ( query , [ ] , [ name : &quot;John&quot; ] , or : [ name : &quot;Alice&quot; , age : 42 ] , or : [ name : &quot;Bob&quot; ] ) QueryBuilder.maybe_where/3 and QueryBuilder.maybe_where/5 query |&gt; QueryBuilder . maybe_where ( some_condition , name : &quot;Alice&quot; ) The above will run where/2 if the given condition is met. QueryBuilder.order_by/2 QueryBuilder . order_by ( query , asc : :lastname , asc : :firstname ) QueryBuilder.order_by/3 QueryBuilder . order_by ( query , :articles , asc : :title@articles ) QueryBuilder.preload/2 QueryBuilder . preload ( query , [ role : :permissions , articles : [ :stars , comments : :user ] ] ) QueryBuilder.left_join/4 QueryBuilder . left_join ( query , :articles , title@articles : &quot;Foo&quot; , or : [ title@articles : &quot;Bar&quot; ] ) QueryBuilder.offset/2 QueryBuilder . offset ( query , 10 ) QueryBuilder.limit/2 QueryBuilder . limit ( query , 10 ) QueryBuilder.from_list/2 QueryBuilder . from_list ( query , [ where : [ name : &quot;John&quot; , city : &quot;Anytown&quot; ] , preload : [ articles : :comments ] , order_by : { :articles , asc : :title@articles } , limit : 20 , offset : 10 ] ) The QueryBuilder.from_list/2 function was the main motivation behind the writing of this library. As explained above, it allows to add querying options to the Context functions. Example: defmodule MyApp.Blog do alias MyApp.Blog.Article def get_article_by_id ( id , opts \\\\ [ ] ) do QueryBuilder . where ( Article , id : id ) |&gt; QueryBuilder . from_list ( opts ) |&gt; Repo . one! ( ) end end The function can now be called as follows (for instance, from a Controller): Blog . get_article_by_id ( id , preload : [ :comments ] ) Blog . get_article_by_id ( id , preload : [ :likes ] ) Blog . get_article_by_id ( id , order_by : { :comments , desc : :inserted_at@comments } , preload : [ comments : :user ] )","ref":"readme.html#usage","title":"Query Builder - Usage","type":"extras"},{"doc":"In the event that you want to extend QueryBuilder's functionality to include custom app specific query functions, there's the QueryBuilder.Extension module to facilitate that. You can create a module with your app specific query functionality and use the QueryBuilder.Extension module to inject all QueryBuilder functions into your custom module. Any custom query functions added to your custom module are included in QueryBuilder.from_list/2 . For example: defmodule MyApp.QueryBuilder do use QueryBuilder.Extension defmacro __using__ ( opts ) do quote do require QueryBuilder QueryBuilder . __using__ ( unquote ( opts ) ) end end # Add app specific query functions here... def where_initcap ( query , field , value ) do text_equals_condition = fn field , value , get_binding_fun -&gt; { field , binding } = get_binding_fun . ( field ) Ecto.Query . dynamic ( [ { ^ binding , x } ] , fragment ( &quot;initcap(?)&quot; , ^ value ) == field ( x , ^ field ) ) end query |&gt; where ( &amp; text_equals_condition . ( field , value , &amp;1 ) ) end end defmodule MyApp.Accounts.User do use MyApp.QueryBuilder schema &quot;users&quot; do field :name , :string field :active , :boolean end end defmodule MyApp.Accounts do alias MyApp.QueryBuilder , as : QB def list_users ( opts \\\\ [ ] ) do # Query list can include custom query functions as well: # [where_initcap: {:name, &quot;john&quot;}, where: {:active, true}] MyApp.Accounts.User |&gt; QB . from_list ( opts ) |&gt; Repo . all ( ) end end","ref":"readme.html#extending-query-functions","title":"Query Builder - Extending Query Functions","type":"extras"},{"doc":"With the auto-binding functionality offered by Query Builder, you can specify field comparisons in queries using the special atom value syntax: :&lt;field_name&gt;@self . This way Query Builder understands the intent is to compare fields vs a raw value. For example: users_where_name_matches_nickname = User |&gt; QueryBuilder . where ( { :name , :eq , :nickname@self } ) |&gt; Repo . all ( ) would resolve to a query along the following form: select * from users where users.name = users.nickname","ref":"readme.html#special-considerations","title":"Query Builder - Special Considerations","type":"extras"},{"doc":"Add query_builder for Elixir as a dependency in your mix.exs file: def deps do [ { :query_builder , &quot;~&gt; 1.0.1&quot; } ] end","ref":"readme.html#installation","title":"Query Builder - Installation","type":"extras"},{"doc":"HexDocs documentation can be found at https://hexdocs.pm/query_builder .","ref":"readme.html#hexdocs","title":"Query Builder - HexDocs","type":"extras"}]